
package org.nzdl.service;

import java.io.*;
import java.util.*;
/** jdk 1.3 orb */
import org.omg.CORBA.*;
/** gsdl corba stuff */
import org.nzdl.corba.gsdlInterface.*;
import org.nzdl.util.*;

public class NzdlServiceImpl extends java.lang.Object implements NzdlService {

  private corbaiface m_nzdlServer = null;

  public NzdlServiceImpl( String [] _args ) {
    try {
	System.err.println("before ORB.init");
      // getting a reference to the remote NZDL server
      org.omg.CORBA.ORB orb = ORB.init( _args, null );
      System.err.println("after ORB.init - before IOR");
      String ior = getIorKey();
      System.err.println("ior = " + ior);
	System.err.println("after IOR");
	org.omg.CORBA.Object obj = orb.string_to_object(ior) ;
      System.err.println("after org stringtoobject");
	m_nzdlServer = corbaifaceHelper.narrow(obj);
	System.err.println("after .narrow");
      // basic setup
      if (m_nzdlServer.initialise()) {
	System.err.println("Nzdl init ... YES");
      }
      else {
	System.err.println("Nzdl init ... NO");
	System.err.println("Aborting service ...");
	//System.exit(1);
      }
    } catch (Exception e) {
      System.err.println("Error in service init:\n" + e) ;
      System.err.println("Aborting service ...");
      System.exit(1);
    }
    System.err.println("Service init ... YES");
  }
  
  public void configure( String _key, Set _values ) {
    m_nzdlServer.configure( NzdlCorbaFactory.toCorbaText( _key ),
			    NzdlCorbaFactory.toCorbaTextArray( _values ));
  }
  
  public Set getCollectionSet( ) {
    corbatext_tarrayHolder c_collSet = NzdlCorbaFactory.createCorbaTextArrayHolder();
    m_nzdlServer.collectionList(c_collSet);
    Set collSet = new HashSet();
    for (int i=0; i<c_collSet.value.length; i++) {
      collSet.add( NzdlCorbaFactory.toString( c_collSet.value[i] ) );
    }
    return collSet ;
  }
  
  public boolean hasCollection( String _name ) {
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    org.omg.CORBA.BooleanHolder c_has = new org.omg.CORBA.BooleanHolder();
    m_nzdlServer.hasCollection( NzdlCorbaFactory.toCorbaText(_name), c_has, c_err );
    if (c_err.value.value() != 0)
      System.err.println("hasCollection error " + c_err.value.value());
    return c_has.value;
  }
  
  public boolean pingCollection( String _name ) {
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    boolean result  = m_nzdlServer.ping( NzdlCorbaFactory.toCorbaText(_name), c_err );
    if (c_err.value.value() != 0)
      System.err.println("pingCollection() error " + c_err.value.value());
    return result;
  }
  
  public NzdlCollectionInfo getCollectionInfo( String _name ) {
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    corbaColInfoResponseHolder c_info = NzdlCorbaFactory.createColInfoResponseHolder();
    m_nzdlServer.getCollectInfo( NzdlCorbaFactory.toCorbaText(_name), c_info, c_err );
    if (c_err.value.value() != 0)
      System.err.println("getCollectionInfo() error " + c_err.value.value());
    return new NzdlCollectionInfo( c_info.value );
  }

  public Set getFilterSet( String _name ) {
    System.err.println("TO BE IMPLEMENTED !!!");
    return null ;
  }

  public Set getMetaTagSet( String _name ) {
    System.err.println("TO BE IMPLEMENTED: DON'T KNOW HOW YET !!!");
    return null ;
  }

  public String getDocument( String _name, String _docID ) {
    corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    corbaDocRequestHolder c_request = NzdlCorbaFactory.createDocRequestHolder( _docID );
    corbaDocResponseHolder c_response = NzdlCorbaFactory.createDocResponseHolder( );
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    m_nzdlServer.getDocument( c_name, c_request, c_response, c_err );
    if (c_err.value.value() != 0)
      System.err.println("getDocument() error " + c_err.value.value());
    return NzdlCorbaFactory.toString( c_response.value.doc );
  }

  public Set getMetaData( String _name, String _docID, String _metaTag ) {
    NzdlRequest request  = new NzdlRequest( _docID, _metaTag );
    NzdlResponse response = new NzdlResponse( );
    service( _name, request, response );
    NzdlResultSet resultSet = response.getResultSet();
    /**
     * Because a result set may contain several hits, we only return
     * the set of metadata values of the specifed meta tag for the
     * specified document ID.
     */
    return resultSet.getMetaData( _docID, _metaTag );
  }

  public Map getMetaData( String _name, List _docIDs, String _metaTag ) {
    NzdlRequest request  = new NzdlRequest( _docIDs, _metaTag );
    NzdlResponse response = new NzdlResponse( );
    service( _name, request, response );
    NzdlResultSet resultSet = response.getResultSet();
    //return resultSet.getMetaData( _docID, _metaTag );
    return resultSet.getMetaData( _metaTag );
    //return resultSet.getAllMetaData( );
  }

  public void service( String _name, NzdlRequest _request, NzdlResponse _response ) {    
    corbatext_t c_name = NzdlCorbaFactory.toCorbaText( _name );
    corbaComErrorHolder c_err = NzdlCorbaFactory.createComErrorHolder();
    m_nzdlServer.filter( c_name, _request.getFilter(), _response.getHolder(), c_err );
    if (c_err.value.value() != 0)
      System.err.println("service() error " + c_err.value.value());
  }

  private String getIorKey() {
    String ior = null;
    try  {
      LineNumberReader input
	= new LineNumberReader(new FileReader("/tmp/localcorba.objid"));
      ior = input.readLine();
    } 
    catch (java.io.IOException e) { 
      System.err.println("Error reading IOR key:\n" + e);
      System.err.println("Aborting service...");
      System.exit(1);
    }
    return ior;
  }

}












